#include <math.h>
#include "pi.h"
#include "self_similar_rayleigh_gans.h"

/* OK so we could do this with the C99 complex type... */
typedef struct {
  double r, i;
} my_complex;

my_complex
dielectric_K(double er, double ei)
{
  my_complex K;
  double denom = er*er + 4*er + 4 + ei*ei;
  K.r = (er*er + er - 2 + ei*ei) / denom;
  K.i = 3 * ei / denom;
  return K;
}

double
dielectric_parameter(double er, double ei)
{
  my_complex K = dielectric_K(er, ei);
  return (K.r*K.r + K.i*K.i);
}

double
self_similar_rayleigh_gans(double Dvol, double axial_ratio, 
			   double refind_r, double refind_i,
			   double fraction, double wavelength,
			   double lambda, double beta, double gamma)
{
  // Dielectric factor
  double K2 = dielectric_parameter(refind_r*refind_r - refind_i*refind_i,
				   2.0*refind_r*refind_i);
  double E2 = 9.0*K2;
  // Wavenumber
  double k = 2.0*M_PI/wavelength;
  double Dz = Dvol*pow(axial_ratio, 2.0/3.0);
  double vol_material = fraction*M_PI*Dvol*Dvol*Dvol/6.0;
  //  double prefactor = (k*k)*(k*k)*M_PI*E2*vol_material*vol_material/(32.0*Dz);
  //  double prefactor = (k*k)*(k*k)*M_PI*E2*vol_material*vol_material/(16.0*Dz*Dz);
  double prefactor = (k*k)*(k*k)*E2*vol_material*vol_material/(64.0*Dz*Dz);
  double term1 = cos(k*Dz)
    *( (1.0+lambda/3.0)*(1.0/(2.0*k+M_PI/Dz) - 1.0/(2.0*k-M_PI/Dz))
       -lambda*(1.0/(2.0*k+3.0*M_PI/Dz) - 1.0/(2.0*k-3.0*M_PI/Dz)) );
  term1 *= term1;
  double sum = 0;
  int jmax = (5.0*k*Dz/M_PI + 1.0);
  int j;
  for (j = 1; j <= jmax; j++) {
    double sin_term = sin(k*Dz);
    double denom1 = 2.0*(k+M_PI*j/Dz);
    double denom2 = 2.0*(k-M_PI*j/Dz);
    sum += pow(2.0*j, -gamma) * sin_term*sin_term
      * (1.0/(denom1*denom1) + 1.0/(denom2*denom2));
  }
  return prefactor*(term1 + beta*sum);
  //return prefactor*term1;
}
